@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Python module: Client ModBus / TCP class 1
-#       Version: 0.0.4
+#       Version: 0.0.5
 #       Website: https://github.com/sourceperl/pyModbusTCP
 #          Date: 2014-08-04
 #       License: MIT (http://http://opensource.org/licenses/mit-license.php)
@@ -10,10 +10,6 @@
 #                1,2,4,5,6 (Class 1)
 #       Charset: utf-8
 
-# TODO
-#   - update the code to deal with IPv6
-#   - add exception ?
-
 from pyModbusTCP import const
 import re
 import socket
@@ -34,7 +30,7 @@ class ModbusClient:
         """
         self.__hostname    = "localhost"
         self.__port        = const.MODBUS_PORT
-        self.__unit_id     = 1                 #
+        self.__unit_id     = 1
         self.__mode        = const.MODBUS_TCP  # default is Modbus/TCP
         self.__sock        = None              # socket handle
         self.__timeout     = 30                # socket timeout
@@ -47,7 +43,7 @@ class ModbusClient:
     def version(self):
         """Get package version
 
-        :return: Current version of the package (like "0.0.1")
+        :return: current version of the package (like "0.0.1")
         :rtype: str
         """
         return self.__version
@@ -69,19 +65,31 @@ class ModbusClient:
         return self.__last_except
 
     def host(self, hostname=None):
-        """Get or set host (IPv4 or hostname like 'plc.domain.net')
+        """Get or set host (IPv4/IPv6 or hostname like 'plc.domain.net')
 
-        :param hostname: hostname/IPv4 address or None for get value
+        :param hostname: hostname or IPv4/IPv6 address or None for get value
         :type hostname: str or None
         :returns: hostname or None if set fail
         :rtype: str or None
         """
-        # return last hostname if no arg
         if hostname is None:
             return self.__hostname
-        # if host is IPv4 address or valid URL
-        if (re.match("^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$", hostname) or
-           (re.match("^[a-z][a-z0-9\.\-]+$", hostname))):
+        # IPv4 ?
+        try:
+            socket.inet_pton(socket.AF_INET, hostname)
+            self.__hostname = hostname
+            return self.__hostname
+        except socket.error:
+            pass
+        # IPv6 ?
+        try:
+            socket.inet_pton(socket.AF_INET6, hostname)
+            self.__hostname = hostname
+            return self.__hostname
+        except socket.error:
+            pass
+        # hostname ?
+        if re.match("^[a-z][a-z0-9\.\-]+$", hostname):
             self.__hostname = hostname
             return self.__hostname
         else:
@@ -103,17 +111,17 @@ class ModbusClient:
         else:
             return None
 
-    def debug(self, debug=None):
+    def debug(self, state=None):
         """Get or set debug mode
 
-        :param debug: debug state or None for get value
-        :type debug: bool or None
+        :param state: debug state or None for get value
+        :type state: bool or None
         :returns: debug state or None if set fail
         :rtype: bool or None
         """
-        if debug is None:
+        if state is None:
             return self.__debug
-        self.__debug = bool(debug)
+        self.__debug = bool(state)
         return self.__debug
 
     def unit_id(self, unit_id=None):
@@ -160,11 +168,11 @@ class ModbusClient:
             self.close()
         # init socket and connect
         # list available sockets on the target host/port
-        # set AF_xxx : AF_INET -> IPv4, AF_INET6 -> IPv6,
-        #              AF_UNSPEC -> IPv6 (priority on some system) or 4
-        # now just accept IPv4: fix this
+        # AF_xxx : AF_INET -> IPv4, AF_INET6 -> IPv6,
+        #          AF_UNSPEC -> IPv6 (priority on some system) or 4
+        # list available socket on target host
         for res in socket.getaddrinfo(self.__hostname, self.__port,
-                                      socket.AF_INET, socket.SOCK_STREAM):
+                                      socket.AF_UNSPEC, socket.SOCK_STREAM):
             af, socktype, proto, canonname, sa = res
             try:
                 self.__sock = socket.socket(af, socktype, proto)
@@ -238,6 +246,12 @@ class ModbusClient:
         # check error
         if not f_body:
             return None
+        # check min frame body size
+        if len(f_body) < 2:
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("read_coils(): rx frame under min size")
+            self.close()
+            return None
         # register extract
         rx_byte_count = struct.unpack("B", f_body[0:1])
         # frame with bits value -> bits[] list
@@ -282,6 +296,12 @@ class ModbusClient:
         # check error
         if not f_body:
             return None
+        # check min frame body size
+        if len(f_body) < 2:
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("read_discrete_inputs(): rx frame under min size")
+            self.close()
+            return None
         # register extract
         rx_byte_count = struct.unpack("B", f_body[0:1])
         # frame with regs value
@@ -325,6 +345,13 @@ class ModbusClient:
         # check error
         if not f_body:
             return None
+        # check min frame body size
+        if len(f_body) < 2:
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("read_holding_registers(): "+
+                             "rx frame under min size")
+            self.close()
+            return None
         # register extract
         rx_reg_count = struct.unpack("B", f_body[0:1])
         # frame with regs value
@@ -332,7 +359,7 @@ class ModbusClient:
         # split f_regs in 2 bytes blocs
         registers = [f_regs[i:i+2] for i in range(0, len(f_regs), 2)]
         registers = [struct.unpack(">H", i)[0] for i in registers]
-        return registers
+        return registers[:int(reg_nb)]
 
     def read_input_registers(self, reg_addr, reg_nb=1):
         """Modbus function READ_INPUT_REGISTERS (0x04)
@@ -367,6 +394,12 @@ class ModbusClient:
         # check error
         if not f_body:
             return None
+        # check min frame body size
+        if len(f_body) < 2:
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("read_input_registers(): rx frame under min size")
+            self.close()
+            return None
         # register extract
         rx_reg_count = struct.unpack("B", f_body[0:1])
         # frame with regs value
@@ -374,7 +407,7 @@ class ModbusClient:
         # split f_regs in 2 bytes blocs
         registers = [f_regs[i:i+2] for i in range(0, len(f_regs), 2)]
         registers = [struct.unpack(">H", i)[0] for i in registers]
-        return registers
+        return registers[:int(reg_nb)]
 
     def write_single_coil(self, bit_addr, bit_value):
         """Modbus function WRITE_SINGLE_COIL (0x05)
@@ -405,6 +438,12 @@ class ModbusClient:
         # check error
         if not f_body:
             return None
+        # check fix frame size
+        if len(f_body) != 4:
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("write_single_coil(): rx frame size error")
+            self.close()
+            return None
         # register extract
         (rx_bit_addr, rx_bit_value, rx_padding) = struct.unpack(">HBB",
                                                                 f_body[:4])
@@ -442,6 +481,12 @@ class ModbusClient:
         # check error
         if not f_body:
             return None
+        # check fix frame size
+        if len(f_body) != 4:
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("write_single_register(): rx frame size error")
+            self.close()
+            return None
         # register extract
         rx_reg_addr, rx_reg_value = struct.unpack(">HH", f_body)
         # check register write
@@ -498,6 +543,12 @@ class ModbusClient:
         # check error
         if not f_body:
             return None
+        # check fix frame size
+        if len(f_body) != 4:
+            self.__last_error = const.MB_RECV_ERR
+            self.__debug_msg("write_multiple_registers(): rx frame size error")
+            self.close()
+            return None
         # register extract
         (rx_reg_addr, rx_reg_nb) = struct.unpack(">HH", f_body[:4])
         # check regs write
@@ -590,7 +641,7 @@ class ModbusClient:
         ## receive
         # modbus TCP receive
         if self.__mode == const.MODBUS_TCP:
-            # 7 bytes header
+            # 7 bytes header (mbap)
             rx_buffer = self._recv(7)
             # check recv
             if not (rx_buffer and len(rx_buffer) == 7):
@@ -613,7 +664,9 @@ class ModbusClient:
                 return None
             # end of frame
             rx_buffer = self._recv(rx_hd_length-1)
-            if not (rx_buffer and (len(rx_buffer) == rx_hd_length-1)):
+            if not (rx_buffer and
+                    (len(rx_buffer) == rx_hd_length-1) and
+                    (len(rx_buffer) >= 2)):
                 self.__last_error = const.MB_RECV_ERR
                 self.__debug_msg("_recv frame body error")
                 self.close()
@@ -627,20 +680,37 @@ class ModbusClient:
             f_body = rx_buffer[1:]
         # modbus RTU receive
         elif self.__mode == const.MODBUS_RTU:
-            rx_buffer = self._recv(const.FRAME_RTU_MAXSIZE)
+            # receive modbus RTU frame (max size is 256 bytes)
+            rx_buffer = self._recv(256)
+            # on _recv error
             if not rx_buffer:
                 return None
             rx_frame = rx_buffer
             # dump frame
             if self.__debug:
                 self._pretty_dump('Rx', rx_frame)
+            # RTU frame min size is 5 bytes
+            if len(rx_buffer) < 5:
+                self.__last_error = const.MB_RECV_ERR
+                self.__debug_msg("short frame error")
+                self.close()
+                return None
+            # check CRC
+            if not self._crc_is_ok(rx_frame):
+                self.__last_error = const.MB_CRC_ERR
+                self.__debug_msg("CRC error")
+                self.close()
+                return None
             # body decode
             (rx_unit_id, rx_bd_fc) = struct.unpack("BB", rx_frame[:2])
-            f_body = rx_frame[2:]
             # check
             if not (rx_unit_id == self.__unit_id):
-              self.close()
-              return None
+                self.__last_error = const.MB_RECV_ERR
+                self.__debug_msg("unit ID mismatch error")
+                self.close()
+                return None
+            # format f_body: remove unit ID, function code and CRC 2 last bytes
+            f_body = rx_frame[2:-2]
         # check except
         if rx_bd_fc > 0x80:
             # except code
@@ -719,8 +789,8 @@ class ModbusClient:
         :rtype: int
         """
         crc = 0xFFFF
-        for index, item in enumerate(frame):
-            next_byte = ord(item)
+        for index, item in enumerate(bytearray(frame)):
+            next_byte = item
             crc ^= next_byte
             for i in range(8):
                 lsb = crc & 1
